# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'Splash_Screen.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PySide2 import QtCore, QtGui, QtWidgets
from .fonts import *
import os


class Ui_SplashScreen(object):
    def setupUi(self, SplashScreen):
        SplashScreen.setObjectName("SplashScreen")
        SplashScreen.resize(804, 403)
        self.centralwidget = QtWidgets.QWidget(SplashScreen)
        self.centralwidget.setMinimumSize(QtCore.QSize(800, 400))
        self.centralwidget.setMaximumSize(QtCore.QSize(800, 400))
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setContentsMargins(3, 3, 3, 3)
        self.verticalLayout.setSpacing(3)
        self.verticalLayout.setObjectName("verticalLayout")
        self.DropShadowFrame = QtWidgets.QFrame(self.centralwidget)
        self.fontDB = QtGui.QFontDatabase()
        self.fontDB.addApplicationFont(f":/fonts/fonts/CabinSketch-Bold.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/CabinSketch-Regular.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/CFNightofTerrorPERSONAL-Reg.ttf")
        self.fontDB.addApplicationFont(f":/fonts/fonts/Courgette-Regular.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/DEADLY KILLERS.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/FEASFBI_.ttf")
        self.fontDB.addApplicationFont(f":/fonts/fonts/FEASFBI.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/FEASFBRG.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/Lemon-Regular.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/Martyric_PersonalUse.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/monbaiti.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/PirataOne-Regular.ttf")
        self.fontDB.addApplicationFont(
                ":/fonts/fonts/RW-creepsville.ttf")
        font = QtGui.QFont()
        font.setFamily("MS PGothic")
        font.setPointSize(15)
        font.setBold(True)
        font.setWeight(75)
        self.DropShadowFrame.setFont(font)
        self.DropShadowFrame.setStyleSheet("QFrame {\n"
"    background-color: rgb(53, 4, 10);\n"
"    color: rgb(252, 72, 90);\n"
"    border-radius: 12px\n"
"}")
        self.DropShadowFrame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.DropShadowFrame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.DropShadowFrame.setObjectName("DropShadowFrame")
        self.AppName = QtWidgets.QLabel(self.DropShadowFrame)
        self.AppName.setGeometry(QtCore.QRect(0, 20, 791, 111))
        font = QtGui.QFont()
        font.setFamily("Creepsville")
        font.setPointSize(45)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        font.setKerning(True)
        font.setStyleStrategy(QtGui.QFont.PreferAntialias)
        self.AppName.setFont(font)
        self.AppName.setStyleSheet("color: rgb(159, 8, 23)")
        self.AppName.setAlignment(QtCore.Qt.AlignCenter)
        self.AppName.setObjectName("AppName")
        self.AppDesc = QtWidgets.QLabel(self.DropShadowFrame)
        self.AppDesc.setGeometry(QtCore.QRect(0, 120, 801, 41))
        font = QtGui.QFont()
        font.setFamily("CF Night of Terror PERSONAL")
        font.setPointSize(14)
        font.setBold(False)
        font.setItalic(True)
        font.setWeight(50)
        font.setKerning(True)
        font.setStyleStrategy(QtGui.QFont.PreferAntialias)
        self.AppDesc.setFont(font)
        self.AppDesc.setStyleSheet("color: rgba(135, 42, 70, 220);")
        self.AppDesc.setAlignment(QtCore.Qt.AlignCenter)
        self.AppDesc.setObjectName("AppDesc")
        self.ProgressBar = QtWidgets.QProgressBar(self.DropShadowFrame)
        self.ProgressBar.setGeometry(QtCore.QRect(70, 270, 671, 23))
        self.ProgressBar.setStyleSheet("QProgressBar {\n"
"    color: rgb(214, 220, 210);\n"
"    background-color: rgba(85, 6, 16, 200);\n"
"    border-style: none;\n"
"    border: 5px qlineargradient(spread:pad, x1:0, y1:0, x2:1, y2:0, stop:0 rgba(158, 7, 23, 255), stop:1 rgba(255, 130, 20, 255));\n"
"    border-radius: 10px;\n"
"    text-align: center;\n"
"}\n"
"\n"
"QProgressBar::chunk {\n"
"    border-radius: 10px;\n"
"    background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:1, y2:0, stop:0 rgba(253, 128, 20, 255), stop:1 rgba(159, 8, 23, 255));\n"
"}")
        self.ProgressBar.setProperty("value", 24)
        self.ProgressBar.setObjectName("ProgressBar")
        self.LoadingStatus = QtWidgets.QLabel(self.DropShadowFrame)
        self.LoadingStatus.setGeometry(QtCore.QRect(0, 300, 791, 21))
        font = QtGui.QFont()
        font.setFamily("CF Night of Terror PERSONAL")
        font.setPointSize(12)
        font.setBold(False)
        font.setItalic(False)
        font.setWeight(50)
        font.setKerning(True)
        font.setStyleStrategy(QtGui.QFont.PreferAntialias)
        self.LoadingStatus.setFont(font)
        self.LoadingStatus.setStyleSheet("color: qlineargradient(spread:pad, x1:0.248473, y1:0.483, x2:1, y2:0, stop:0.208955 rgba(131, 62, 40, 231), stop:1 rgba(163, 13, 23, 248));")
        self.LoadingStatus.setAlignment(QtCore.Qt.AlignCenter)
        self.LoadingStatus.setObjectName("LoadingStatus")
        self.CreatedBy = QtWidgets.QLabel(self.DropShadowFrame)
        self.CreatedBy.setGeometry(QtCore.QRect(320, 370, 461, 16))
        font = QtGui.QFont()
        font.setFamily("Comic Sans MS")
        font.setPointSize(8)
        font.setBold(True)
        font.setItalic(False)
        font.setWeight(75)
        font.setKerning(True)
        font.setStyleStrategy(QtGui.QFont.PreferAntialias)
        self.CreatedBy.setFont(font)
        self.CreatedBy.setStyleSheet("color: qlineargradient(spread:pad, x1:0.248473, y1:0.483, x2:1, y2:0, stop:0.208955 rgba(131, 62, 40, 231), stop:1 rgba(163, 13, 23, 248));\n"
"border-radius: 100px;")
        self.CreatedBy.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.CreatedBy.setObjectName("CreatedBy")
        self.Version = QtWidgets.QLabel(self.DropShadowFrame)
        self.Version.setGeometry(QtCore.QRect(10, 0, 55, 16))
        self.Version.setStyleSheet("border: 0px solid;\n"
"border-radius: 10px;")
        self.Version.setObjectName("Version")
        self.verticalLayout.addWidget(self.DropShadowFrame)
        SplashScreen.setCentralWidget(self.centralwidget)

        self.retranslateUi(SplashScreen)
        QtCore.QMetaObject.connectSlotsByName(SplashScreen)

    def retranslateUi(self, SplashScreen):
        _translate = QtCore.QCoreApplication.translate
        SplashScreen.setWindowTitle(_translate("SplashScreen", "MainWindow"))
        self.AppName.setText(_translate("SplashScreen", "Thor<strong>𝜓vos</strong>"))
        self.AppDesc.setText(_translate("SplashScreen", "The all in one cryptographic toolkit"))
        self.LoadingStatus.setText(_translate("SplashScreen", "Loading..."))
        self.CreatedBy.setText(_translate("SplashScreen", "Made By: Rakshan Sharma"))
        self.Version.setText(_translate("SplashScreen", "V1.0"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    SplashScreen = QtWidgets.QMainWindow()
    ui = Ui_SplashScreen()
    ui.setupUi(SplashScreen)
    SplashScreen.show()
    sys.exit(app.exec_())



"""Converts files to WAV files and vice versa."""
import wave


# Converts any file to WAV with a .wav header
def file2wave(infile, outfile):
    """Take input a file, any format, and gives output a WAV file."""
    with open(infile, "rb") as input_file:
        with wave.open(outfile, "wb") as output_file:
            output_file.setnchannels(1)
            output_file.setsampwidth(4)
            output_file.setframerate(96000*4)
            data = input_file.read()
            output_file.writeframesraw(data)


# Converts any wav file to an output file
def wave2file(infile, outfile):
    """Take a WAV file as input and convert it into a different file format."""
    with open(outfile, "wb") as output_file:
        with open(infile, "rb") as input_file:
            data = input_file.read()
            output_file.write(data)


# Encodes data into wav file
def encwave(data, outfile):
    """Take input a file, any format, and gives output a WAV file."""
    with wave.open(outfile, "wb") as output_file:
        output_file.setnchannels(1)
        output_file.setsampwidth(4)
        output_file.setframerate(96000*4)
        output_file.writeframesraw(data)


def write_data(data, outfile):
    """Write data to a file."""
    with open(outfile, "wb") as output_file:
        output_file.write(data)


if __name__ == '__main__':
    exit("This is a helper function for thoryvos.")



"""
Steganography Backend for thoryvos.

The LSB algorithm is taken from:
 https://github.com/pavanchhatpar/wav-steg-py
"""
# Importing Dependencies
from os import stat
from math import ceil
from struct import pack, unpack
from array import array

import PyWave
import soundfile as sf


class Stego:
    """Steganographer Class."""

    def __init__(self, infile: str, lsb=None) -> None:
        """Set the required data from the infile."""
        # Opening the files.
        data, samplerate = sf.read(infile)
        self.audio = PyWave.open(infile, 'r')
        sf_object = sf.SoundFile(infile)

        # Saving the parameters of the infile for later.
        self.params = {
            'channels': self.audio.channels,
            'frequency': self.audio.frequency,
            'bitrate': self.audio.bitrate,
            'format': self.audio.format,
            'bits_per_sample': self.audio.bits_per_sample
        }

        # Setting the required parameters for lsb algorithm.
        self.nframes = len(data)
        self.nsamples = self.nframes * sf_object.channels
        self.type = sf_object.subtype
        self.lsb = lsb

        # Closing the soundfile object.
        sf_object.close()

    def prepeare_hide(self, datafile: str) -> None:
        """Compatibility Check & Sets required parameters."""
        # Getting the size of the datafile
        self.datasize = stat(datafile).st_size

        # If we are not provided with a lsb, we set it to optimal value.
        if not self.lsb:
            self.lsb = self.get_lsb()

        # Required for the lsb algorithm.
        self.fmt, self.mask, self.smallest_byte = self.get_req_data()

        # Checking if there is enough space in the infile to hide data.
        max_bytes = (self.nsamples * self.lsb) // 8
        if self.datasize > max_bytes:
            self.raise_exception()

    def raise_exception(self) -> None:
        """Raise errors."""
        # Getting the minimum number of lsb required
        req_lsb = self.get_lsb()
        # Since, there are only 8 lsb in the infile
        if req_lsb >= 8:
            raise ValueError("Datafile is too large for this audio file.")

        raise ValueError(f"{req_lsb} required to hide datafile in audio file")

    def hide(self, datafile: str, outfile: str) -> tuple:
        """Hide data in the audio file."""
        # Setting the required parameters for hiding the data.
        self.prepeare_hide(datafile)

        # Storing the plain audio data for processing in an array.
        plain_audio_data = array('i', unpack(self.fmt, self.audio.read()))
        self.audio.close()

        # Getting a pointer to the buffer to the data
        data = memoryview(open(datafile, 'rb').read())

        # Keeps track of position in the audio & data file.
        data_index = 0
        audio_index = 0

        # Initializing an array to store the stego data.
        stego_audio_data = []

        # Initializng some more variables for the lsb algorithm.
        buffer = 0
        buffer_length = 0
        complete = False

        # The actual lsb algorithm.
        while not complete:
            while (buffer_length < self.lsb and data_index // 8 < len(data)):

                buffer += (data[data_index // 8] >> (data_index % 8)
                           ) << buffer_length
                data_added = 8 - (data_index % 8)
                buffer_length += data_added
                data_index += data_added

            current_bit = buffer % (1 << self.lsb)
            buffer >>= self.lsb
            buffer_length -= self.lsb

            while (audio_index < len(plain_audio_data) and
                   plain_audio_data[audio_index] == self.smallest_byte):

                stego_audio_data.append(pack(self.fmt[-1],
                                             plain_audio_data[audio_index]))
                audio_index += 1

            if audio_index < len(plain_audio_data):
                current_sample = plain_audio_data[audio_index]
                audio_index += 1

                sign = 1

                if current_sample < 0:
                    current_sample = -current_sample
                    sign = -1

                stego_sample = sign * ((current_sample & self.mask) |
                                       current_bit)
                stego_audio_data.append(pack(self.fmt[-1], stego_sample))

            if (data_index // 8 >= len(data) and buffer_length <= 0):
                complete = True

        # Append the rest of the data from plain audio to stego audio
        while audio_index < len(plain_audio_data):
            stego_audio_data.append(pack(self.fmt[-1],
                                         plain_audio_data[audio_index]))
            audio_index += 1

        # Save stego file.
        with PyWave.open(outfile,
                         mode="w",
                         channels=self.params["channels"],
                         frequency=self.params["frequency"],
                         bits_per_sample=self.params["bits_per_sample"],
                         format=self.params["format"]) as out_file:
            out_file.write(b"".join(stego_audio_data))

        # Needed for retrieval.
        return (self.lsb, self.datasize)

    def prepeare_recover(self) -> None:
        """Set the parameters required for retrieval."""
        # The default lsb value for retrieval is 1
        if not self.lsb:
            self.lsb = 1

        # Required for the lsb recovery algorithm.
        self.fmt, _, self.smallest_byte = self.get_req_data()

        self.mask = (1 << self.lsb) - 1

    def recover(self, outfile: str, nbytes: int) -> bool:
        """Recover data from the audio file."""
        # Setting the required parameters for retrieval the data.
        self.prepeare_recover()

        # Storing the audio data in an array for processing.
        stego_audio_data = array('i', unpack(self.fmt, self.audio.read()))
        self.audio.close()

        # Initializng the variables and data structures for retrieval.
        data = bytearray()
        index = 0
        buffer = 0
        buffer_length = 0

        # The actual lsb algorithm.
        while nbytes > 0:
            current_sample = stego_audio_data[index]

            if current_sample != self.smallest_byte:

                buffer += (abs(current_sample) & self.mask) << buffer_length
                buffer_length += self.lsb

            index += 1

            while (buffer_length >= 8 and nbytes > 0):

                current_sample = buffer % (1 << 8)
                buffer >>= 8
                buffer_length -= 8
                data += pack('1B', current_sample)
                nbytes -= 1

        # Writing data to the file.
        with open(outfile, 'wb+') as out_file:
            out_file.write(bytes(data))

        return True

    def get_lsb(self) -> int:
        """Get the optimal lsb for the data & audio file."""
        return ceil(self.datasize * 8 / self.nsamples)

    def get_req_data(self) -> tuple:
        """Return the fmt, mask & smallest byte based on the filetype."""
        if self.type == "PCM_U8":
            """Samples are unsigned 8-bit integers."""
            fmt = f"{self.nsamples}B"
            mask = (1 << 8) - (1 << self.lsb)
            smallest_byte = -(1 << 8)

        elif self.type == "PCM_S8":
            """Samples are signed 8-bit integers."""
            fmt = f"{self.nsamples}b"
            mask = (1 << 8) - (1 << self.lsb)
            smallest_byte = -(1 << 8)

        elif self.type == 'PCM_16':
            """Samples are signed 16-bit integers."""
            fmt = f"{self.nsamples}h"
            mask = (1 << 15) - (1 << self.lsb)
            smallest_byte = -(1 << 15)

        elif self.type == 'PCM_24':
            """Samples are signed 24-bit integers."""
            fmt = f"{self.nsamples * 3}B"
            mask = (1 << 23) - (1 << self.lsb)
            smallest_byte = -(1 << 23)

        elif self.type == 'PCM_32':
            """Samples are signed 32-bit integers."""
            fmt = f"{self.nsamples}i"
            mask = (1 << 31) - (1 << self.lsb)
            smallest_byte = -(1 << 31)

        elif self.type == 'PCM_64':
            """Samples are signed 64-bit integers."""
            fmt = f"{self.nsamples}q"
            mask = (1 << 63) - (1 << self.lsb)
            smallest_byte = -(1 << 63)

        else:
            raise Exception(f"Unsupported WAV file.\
                            {self.type} is not supported.")
            exit(1)

        return (fmt, mask, smallest_byte)


if __name__ == '__main__':
    exit("""

          DIRECT COMMAND LINE INTERFACE NOT AVAILABLE
          USE thoryvos_cli.py INSTEAD

          To use as a module:
            import thoryvos_stego as st
            x = st.Stego(infile, lsb)
            x.hide(datafile, outfile)

            y = st.Stego(infile, lsb)
            y.recover(datafile, bytes_hidden)
          """)



"""Command Line Interface for thoryvos."""
from termcolor import cprint
from os import get_terminal_size
from .thoryvos_driver import encryptor, decryptor, hide_data, recover_data, anon_upload, anon_download
import argparse
from .thoryvos_errors import *


def credits():
    """Print Credits."""
    size = get_terminal_size()

    cprint('thor\N{MATHEMATICAL ITALIC SMALL PSI}vos'.center(
        size.columns, " "), 'red', attrs=['bold'])
    cprint("The all-in-one cryptography toolkit".center(size.columns,
                                                                   " "), 'red', attrs=['dark'])
    cprint("="*(size.columns-4), attrs=['dark'])
    cprint("Made by Rakshan Sharma, V1.0", 'blue')
    cprint("Use '-h' flag to get help.")
    cprint("="*(size.columns-4), attrs=['dark'])
    print()


def parse_cl():
    """CLI Argument Parser."""
    parser = argparse.ArgumentParser(description='thoryvos Command Line\
                                     Interface. Macros are available on GUI.')
    parser.add_argument('-i', '--infile',
                        help="The input filename.",
                        type=str,
                        required=False)
    parser.add_argument('-o', '--outfile',
                        help="The output filename.",
                        type=str,
                        required=False)
    parser.add_argument('-m', '--mode', '-operation', '-op',
                        help="Available Modes: encrypt/decrypt/hide/recover/\
                        upload/download",
                        type=str,
                        required=True)
    parser.add_argument('-u', '--url',
                        help="Anonfiles URL to download",
                        type=str,
                        required=False)
    parser.add_argument('-a', '--algorithm', '--algo',
                        help="The algorithm for encryption/decryption.\
                        Available algorithms are: AES/DES/Salsa20",
                        type=str,
                        required=False)
    parser.add_argument('-p', '--password',
                        help="Password",
                        type=str,
                        required=False)
    parser.add_argument('-df', '--datafile', '-d',
                        help="The data filename.",
                        type=str,
                        required=False)
    parser.add_argument('-l', '--lsb',
                        help="LSB used during hiding.",
                        type=int,
                        required=False)
    parser.add_argument('-nb', '--nbytes', '-n',
                        help="Number of bytes hidden.",
                        type=int,
                        required=False)

    args = parser.parse_args()
    credits()

    if args.mode == 'encrypt':
        if not args.password:
            exit("Password is required.")
        if not args.algorithm:
            raise InvalidEncryptionMode
        if not args.infile:
            raise FileDoesNotExist
        if not args.outfile:
            exit("Output file is required.")

        encryptor(args.infile, args.outfile, args.password,
                           args.algorithm.upper())

        exit("Successfully Ecrypted.")

    if args.mode == 'decrypt':
        if not args.password:
            exit("Password is required.")
        if not args.algorithm:
            raise InvalidEncryptionMode
        if not args.infile:
            raise FileDoesNotExist
        if not args.outfile:
            exit("Output file is required.")

        decryptor(args.infile, args.outfile, args.password,
                           args.algorithm.upper())

        exit("Successfully Decrypted.")

    if args.mode == 'hide':
        if not args.datafile:
            raise EmptyDataFile
        if not args.lsb:
            args.lsb = None
        if not args.infile:
            raise FileDoesNotExist
        if not args.outfile:
            exit("Output file is required.")

        hide_data(args.infile, args.outfile,
                           args.datafile, args.lsb)

        cprint(
    f"""YOU MUST REMEMBER THESE VALUES FOR DATA EXTRACTION:
    LSB: {code[0]}
    Bytes To Recover: {code[1]}""", 'red', attrs=['bold'])

        exit(0)

    if args.mode == 'recover':
        if not args.lsb:
            raise LSBError
        if not args.nbytes:
            raise NBytesError
        if not args.infile:
            raise FileDoesNotExist
        if not args.outfile:
            exit("Output file is required.")

        recover_data(args.infile, args.outfile, args.lsb,
                              args.nbytes)

        exit(0)

    if args.mode == 'upload':
        if not args.infile:
            raise FileDoesNotExist

        URL = anon_upload(args.infile)

        exit(f"URL: {URL}")

    if args.mode == 'download':
        if not args.url:
            raise URLError

        filename = anon_download(args.url)

        exit(f"Saved to {filename}")


if __name__ == '__main__':
    parse_cl()



"""throyvos Driver Code."""
# Importing backend libraries
from .thoryvos_encoder import write_data
from .thoryvos_crypto import Encrypt, Decrypt
from .thoryvos_stego import Stego
from .thoryvos_transfer import upload, download, verify
from .thoryvos_errors import *
from Crypto.Util.Padding import pad

# Importing Dependencies
from os.path import exists, splitext
from os import remove


def cleanup(*args, **kwargs):
    """Delete created files in case of error."""
    for file in args:
        if exists(file):
            remove(file)
    for file in kwargs:
        if exists(file):
            remove(file)


def get_extension(filename):
    """Return extension padded tp 8 bytes in binary format."""
    extension = splitext(filename)[1].encode()
    return pad(extension, 16)


def encryptor(infile: str, outfile: str, password: str, mode: str) -> int:
    """Encryption Driver for thoryvos backend."""
    enc = Encrypt(infile)

    if mode.upper() == 'AES':
        encrypted_data = enc.AES(password)
    elif mode.upper() == 'DES':
        encrypted_data = enc.DES(password)
    elif mode.upper() == 'SALSA20':
        encrypted_data = enc.Salsa20(password)
    else:
        raise InvalidEncryptionMode

    if not encrypted_data:
        raise EmptyDataFile

    write_data(get_extension(infile) + encrypted_data, outfile)


def decryptor(infile: str, outfile: str, password: str, mode: str) -> int:
    """Decryption Driver for thoryvos backend."""

    dec = Decrypt(infile)

    if mode.upper() == 'AES':
        decrypted_data = dec.AES(password)
    elif mode.upper() == 'DES':
        decrypted_data = dec.DES(password)
    elif mode.upper() == 'SALSA20':
        decrypted_data = dec.Salsa20(password)
    else:
        raise InvalidEncryptionMode

    if not decrypted_data:
        cleanup(outfile)
        raise EmptyDataFile

    if not outfile.endswith(dec.extension):
        outfile += dec.extension
    write_data(decrypted_data[16:], outfile)


def anon_upload(infile: str):
    """File Transfer Driver for throyvos backend."""
    if exists(infile):
        URL = upload(infile)
        return URL
    raise URLError


def anon_download(url: str):
    """File Transfer Driver for throyvos backend."""
    if verify(url):
        location = download(url)
        return location
    raise FileDoesNotExist


def hide_data(infile: str, outfile: str, datafile: str, lsb=None):
    """Steganography Hiding Driver for throyvos backend."""
    if not infile.endswith('.wav'):
        raise NotAWAVFile

    if not outfile.endswith('.wav'):
        raise NotAWAVFile

    try:
        lsb = int(lsb)
    except TypeError:
        lsb = None

    steganographer = Stego(infile, lsb)
    lsb, datasize = steganographer.hide(datafile, outfile)

    if not datasize:
        cleanup(outfile)
        raise EmptyDataFile

    return (lsb, datasize)


def recover_data(infile: str, outfile: str, lsb: int, nbytes: int):
    """Steganography Retrieval Driver for throyvos backend."""
    if not lsb:
        raise LSBError
    if not nbytes:
        raise NBytesError

    if not infile.endswith('.wav'):
        raise NotAWAVFile

    try:
        lsb = int(lsb)
    except TypeError:
        lsb = lsb

    steganographer = Stego(infile, lsb)
    if steganographer.recover(outfile, nbytes):
        return

    cleanup(outfile)
    raise UnexpectedError("Error... CleanUp...OK")


if __name__ == '__main__':
    exit("Driver code cannot be used directly.")



class UnexpectedError(Exception):
    """Uneforseen Error."""
    def __init__(self, message=None):
        ErrorMsg = ("An Unexpected Error has occured.")
        self.message = (ErrorMsg + " " + message) if message is not None else ErrorMsg
        super().__init__(self.message)


class InvalidEncryptionMode(Exception):
    """The encryption mode entered is invalid."""
    
    def __init__(self, mode):
        self.ErrorMsg = f"Unsupported Encryption/Decryption Mode: {mode}"
        AvailableModes = "AES, DES, Salsa20"
        super().__init__(self.ErrorMsg + "\n Currently Supported Modes are: (Case Insensitive): \n  " + AvailableModes)


class EmptyDataFile(Exception):
    """The Data File provided is empty."""
    def __init__(self, path=None):
        self.ErrorMsg = "Data File Provided is empty." if not path else f"{path} is empty."
        super().__init__(self.ErrorMsg)


class NotAWAVFile(Exception):
    """The file provided is not a WAV file."""
    def __init__(self, path=None):
        self.ErrorMsg = "The file provided must be a WAV file." if not path else f"{path} is not a WAV File."
        super().__init__(self.ErrorMsg)


class URLError(Exception):
    """The URL provided is invalid."""
    def __init__(self):
        super().__init__("Invalid URL.")


class FileDoesNotExist(Exception):
    """The path provided is invalid."""
    def __init__(self, path=None):
        self.ErrorMsg = "File does not exist." if not path else f"{path} is invalid."
        super().__init__(self.ErrorMsg)
    

class LSBError(Exception):
    """LSB must be specified for extraction."""
    def __init__(self):
        super().__init__("The LSB value must be specified for extraction/retrieval.")


class NBytesError(Exception):
    """NBytes must be specified during extraction/retrieval."""
    def __init__(self):
        super().__init__("The number of bytes to extract must be specified for extraction/retrieval.")

if __name__ == "__main__":
    raise NBytesError()



from .thoryvos import *


class UIFunctions(MainWindow):
    """Animation."""
    def toggleMenu(self, maxWidth, enable):
        if not enable:
            return
        # Get Current Width
        Width = self.ui.LeftMenu.width()
        default = 80

        # Switch b/w open & close
        if Width == default:
            NewWidth = maxWidth
        else:
            NewWidth = default

        # Animation
        self.animation = QPropertyAnimation(self.ui.LeftMenu,
                                            b"minimumWidth")
        self.animation.setDuration(500)  # Animation Duration (in ms)
        self.animation.setStartValue(Width)
        self.animation.setEndValue(NewWidth)
        self.animation.setEasingCurve(QtCore.QEasingCurve.InOutQuart)
        self.animation.start()



"""Transfer files to/from anonfiles.com using AnonFile API."""
from anonfile.anonfile import AnonFile
from os import getcwd


def upload(infile):
    """Upload the output file to anonfiles and returns a link."""
    anon = AnonFile()
    status, file_url = anon.upload_file(infile)
    return file_url


def download(url):
    """Download a file from anonfiles."""
    anon = AnonFile()
    location = getcwd()
    anon.download_file(url, location)
    return location


def verify(text):
    """Verify that the url is valid in a very simple manners."""
    if 'anonfiles.com' in text:
        return True
    return False



from .UI_Functions import *
from .thoryvos import *
from .thoryvos_cli import parse_cl
import sys


def gui():
    app = QApplication(sys.argv)
    window = SplashScreen()
    sys.exit(app.exec_())


def main():
    if len(sys.argv) == 1:
        gui()
    else:
        parse_cl()

if __name__ == "__main__":
    main()



# Importing Required Modules
import sys
import platform
from PySide2 import QtCore, QtWidgets, QtGui
from PySide2.QtCore import *
from PySide2.QtWidgets import *
from PySide2.QtGui import *
import os
from .fonts import *
from time import sleep

# GUI
from .main_ui import Ui_MainWindow
from .splash_screen_ui import Ui_SplashScreen

# Import UI Functions
from .UI_Functions import *
from .StyleSheets import *

# Importing thoryvos backend
from .thoryvos_driver_gui import recover_data, decryptor, encryptor, hide_data, anon_download, anon_upload, verify
from .thoryvos_errorcodes import *

# Definign Global Variables
Progress = 0
Infile = None
Datafile = None
Outfile = None
URL = None
ERRORCODES = range(1, 9)


class MainWindow(QMainWindow):
    """The main Window in the GUI Interface."""

    def __init__(self):
        """Configure the GUI interface."""
        QMainWindow.__init__(self)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.ui.AppDesc.setText("Welcome...")

        self.fontDB = QFontDatabase()
        self.fontDB.addApplicationFont(f":/fonts/fonts/CabinSketch-Bold.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/CabinSketch-Regular.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/CFNightofTerrorPERSONAL-Reg.ttf")
        self.fontDB.addApplicationFont(f":/fonts/fonts/Courgette-Regular.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/DEADLY KILLERS.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/FEASFBI_.ttf")
        self.fontDB.addApplicationFont(f":/fonts/fonts/FEASFBI.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/FEASFBRG.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/Lemon-Regular.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/Martyric_PersonalUse.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/monbaiti.ttf")
        self.fontDB.addApplicationFont(
            f":/fonts/fonts/PirataOne-Regular.ttf")
        self.fontDB.addApplicationFont(
            ":/fonts/fonts/RW-creepsville.ttf")

        self.lsb = None
        self.nb = None

        # Toggle Menu Bar
        self.ui.ToggleMenu.clicked.connect(
            lambda: UIFunctions.toggleMenu(self, 180, True))
        QtCore.QTimer.singleShot(1500, lambda: self.ui.AppDesc.setText(
            "The all in one cryptographic toolkit"))

        # Left Menu Configuration
        self.ui.Home.clicked.connect(
            lambda: self.ui.Stack.setCurrentWidget(self.ui.HomePage))
        self.ui.Crypto.clicked.connect(
            lambda: self.ui.Stack.setCurrentWidget(self.ui.CryptoPage))
        self.ui.MarosMenuButton.clicked.connect(
            lambda: self.ui.Stack.setCurrentWidget(self.ui.MacroPage))
        self.ui.FileShare.clicked.connect(
            lambda: self.ui.Stack.setCurrentWidget(self.ui.TransferPage))
        self.ui.Stego.clicked.connect(
            lambda: self.ui.Stack.setCurrentWidget(self.ui.StegoPage))
        self.ui.Stego.clicked.connect(
            lambda: self.ui.StegoPage_2.setCurrentWidget(self.ui.StegoOptions))
        self.ui.MarosMenuButton.clicked.connect(
            lambda: self.ui.stackedWidget.setCurrentWidget(self.ui.MacrosPage))
        self.ui.FileShare.clicked.connect(self.reset)
        self.ui.MarosMenuButton.clicked.connect(self.reset)
        self.ui.Home.clicked.connect(self.reset)
        self.ui.Crypto.clicked.connect(self.reset)
        self.ui.Stego.clicked.connect(self.reset)

        # Home Page Buttons Configuration
        self.ui.CryptoPageButton.clicked.connect(
            lambda: self.ui.Stack.setCurrentWidget(self.ui.CryptoPage))
        self.ui.MacroPageButton.clicked.connect(
            lambda: self.ui.Stack.setCurrentWidget(self.ui.MacroPage))
        self.ui.TransferPageButton.clicked.connect(
            lambda: self.ui.Stack.setCurrentWidget(self.ui.TransferPage))
        self.ui.StegoPageButton.clicked.connect(
            lambda: self.ui.Stack.setCurrentWidget(self.ui.StegoPage))
        self.ui.CryptoPageButton.clicked.connect(self.reset)
        self.ui.MacroPageButton.clicked.connect(self.reset)
        self.ui.TransferPageButton.clicked.connect(self.reset)
        self.ui.StegoPageButton.clicked.connect(self.reset)
        self.ui.StegoPageButton.clicked.connect(
            lambda: self.ui.StegoPage_2.setCurrentWidget(self.ui.StegoOptions))
        self.ui.MacroPageButton.clicked.connect(
            lambda: self.ui.stackedWidget.setCurrentWidget(self.ui.MacrosPage))


        # Crypto Page Configuration
        self.ui.CryptoDragDrop.dropped.connect(self.crypto_add_file)
        self.ui.CryptoDragDrop.clicked.connect(self.crypto_add_dialog)

        self.ui.AlgoSelect.addItem("AES")
        self.ui.AlgoSelect.addItem("DES")
        self.ui.AlgoSelect.addItem("SALSA20")

        self.ui.CryptoSaveLocBrowse.clicked.connect(self.crypto_save_file)
        self.ui.Password.textChanged.connect(self.update_password)

        self.ui.Encrypt.clicked.connect(self.encrypt)
        self.ui.Decrypt.clicked.connect(self.decrypt)

        # Transfer Page
        self.ui.DragDropTransfer.dropped.connect(self.transfer_add_file)
        self.ui.DragDropTransfer.clicked.connect(self.transfer_file_dialog)

        self.ui.Url.textChanged.connect(self.update_transfer_label)

        self.ui.ButtonTransfer.clicked.connect(self.transfer)

        # Stego Page
        self.ui.HideButton.clicked.connect(
            lambda: self.ui.StegoPage_2.setCurrentWidget(self.ui.Hide))
        self.ui.HideButton.clicked.connect(self.reset)
        self.ui.RecoverButton.clicked.connect(
            lambda: self.ui.StegoPage_2.setCurrentWidget(self.ui.Recover))
        self.ui.RecoverButton.clicked.connect(self.reset)

        # Hide Page for Stego
        self.ui.InfileDragDrop.dropped.connect(self.stego_add_file)
        self.ui.InfileDragDrop.clicked.connect(self.stego_add_dialog)

        self.ui.DatafileDragDrop.dropped.connect(self.stego_add_datafile)
        self.ui.DatafileDragDrop.clicked.connect(self.stego_add_datadialog)

        self.ui.SaveLocBrowseStego.clicked.connect(self.stego_save_file)
        self.ui.lsb.textChanged.connect(self.update_lsb())
        self.ui.HideBtn.clicked.connect(self.hide)

        # Recover Page for Stego
        self.ui.InfileRedDragDrop.dropped.connect(self.stego_add_file)
        self.ui.InfileRedDragDrop.clicked.connect(self.stego_add_dialog)

        self.ui.OutfileRecBrowse.clicked.connect(self.stego_save_file_rec)
        self.ui.LSB.textChanged.connect(self.update_lsb_rec)
        self.ui.NBytes.textChanged.connect(self.update_nbytes)

        self.ui.RecButton.clicked.connect(self.recover)

        # Macros Page
        self.ui.EncSteg.clicked.connect(
            lambda: self.ui.stackedWidget.setCurrentWidget(self.ui.EncryptStegoPage))
        self.ui.EncSteg.clicked.connect(self.reset)

        # EncSteg Page for Macros
        self.ui.MInfileDragDrop.dropped.connect(self.stego_add_file)
        self.ui.MInfileDragDrop.clicked.connect(self.stego_add_dialog)

        self.ui.MDatafileDragDrop.dropped.connect(self.stego_add_datafile)
        self.ui.MDatafileDragDrop.clicked.connect(self.stego_add_datadialog)

        self.ui.SaveLocationBrowse.clicked.connect(self.stego_save_file_rec)
        self.ui.LSBInput.textChanged.connect(self.update_lsb_cir)
        self.ui.NBytesInput.textChanged.connect(self.update_nbytes_cir)

        self.ui.Forward.clicked.connect(self.circuit_1)
        self.ui.Backward.clicked.connect(self.circuit_1_backward)

        self.show()

    def circuit_1_backward(self):
        """Macro Option 1 Backward Driver Code."""
        global Infile, Outfile

        password, start = QInputDialog.getText(self, "Password",
                                               "Enter Password:",
                                               QLineEdit.Password,
                                               flags=Qt.FramelessWindowHint)

        if not start:
            return

        steg = recover_data(Infile, "thoryvos_temp2217761.wav", self.lsb, int(self.nb))
        if steg in ERRORCODES:
            self.reset()
            self.ui.MInfileLabel.setStyleSheet("")
            self.ui.MInfileLabel.setText(Error[steg])
            os.remove("thoryvos_temp2217761.wav")
            return

        dec = decryptor("thoryvos_temp2217761.wav", Outfile, password, "AES")
        if dec in ERRORCODES:
            self.reset()
            self.ui.MInfileLabel.setStyleSheet("")
            self.ui.MInfileLabel.setText(f"{Error[dec]}")
            os.remove("thoryvos_temp2217761.wav")
            return

        self.reset()
        os.remove("thoryvos_temp2217761.wav")
        self.ui.MInfileLabel.setText("Circuit Succesfully Executed")

    def circuit_1(self):
        """Macro Option 1 Forward Driver Code."""
        global Infile, Outfile, Datafile

        password, start = QInputDialog.getText(self, "Password",
                                               "Enter Password:",
                                               QLineEdit.Password,
                                               flags=Qt.FramelessWindowHint)

        if not start:
            return

        enc = encryptor(Datafile, "thoryvos_temp2217761.wav", password, "AES")
        if enc in ERRORCODES:
            self.reset()
            self.ui.MInfileLabel.setStyleSheet("")
            self.ui.MInfileLabel.setText(f"{Error[enc]}")
            os.remove("thoryvos_temp2217761.wav")
            return

        steg = hide_data(
            Infile, Outfile, "thoryvos_temp2217761.wav", self.lsb)
        if steg in ERRORCODES:
            self.reset()
            self.ui.MInfileLabel.setStyleSheet("")
            self.ui.MInfileLabel.setText(f"{Error[steg]}")
            os.remove("thoryvos_temp2217761.wav")
            return

        self.reset()
        os.remove("thoryvos_temp2217761.wav")
        self.ui.MInfileLabel.setText("Circuit Succesfully Executed")
        self.ui.MDatafileLabel.setText(f"LSB: {steg[0]} \n NBytes: {steg[1]}")

    def recover(self):
        """Recover Driver Code."""
        global Infile, Outfile

        code = recover_data(Infile, Outfile, self.lsb, int(self.nb))
        self.reset()
        if code in ERRORCODES:
            self.ui.InfileRecLabel.setText(f"{Error[code]}")
        else:
            self.ui.InfileRecLabel.setText("Succesfully Recovered.")

    def update_lsb(self):
        """Update self.lsb according to lsb in the field."""
        lsb = self.ui.lsb.text()
        if lsb != "":
            self.lsb = lsb
        else:
            self.lsb = None

    def update_lsb_cir(self):
        """Update self.lsb according to lsb in the field."""
        lsb = self.ui.LSBInput.text()
        if lsb != "":
            self.lsb = lsb
        else:
            self.lsb = None
            self.ui.Backward.setEnabled(False)
            self.ui.Backward.setText("")

        if self.lsb is not None and self.nb is not None:
            self.ui.Backward.setEnabled(True)
            self.ui.Backward.setText("Backward Circuit [Rec -> Dec]")

    def update_lsb_rec(self):
        """Update self.lsb according to lsb in the field."""
        lsb = self.ui.LSB.text()
        if lsb != "":
            self.lsb = lsb
        else:
            self.lsb = None
            self.ui.RecButton.setEnabled(False)
            self.ui.RecButton.setText("")

        if self.lsb is not None and self.nb is not None:
            self.ui.RecButton.setEnabled(True)
            self.ui.RecButton.setText("Recover")

    def update_nbytes_cir(self):
        """Update self.nb according to nbytes in the field."""
        nb = self.ui.NBytesInput.text()
        if nb != "":
            self.nb = nb
        else:
            self.nb = None
            self.ui.Backward.setEnabled(False)
            self.ui.Backward.setText("")

        if self.lsb is not None and self.nb is not None:
            self.ui.Backward.setEnabled(True)
            self.ui.Backward.setText("Backward Circuit [Rec -> Dec]")

    def update_nbytes(self):
        """Update self.nb according to nbytes in the field."""
        nb = self.ui.NBytes.text()
        if nb != "":
            self.nb = nb
        else:
            self.nb = None
            self.ui.RecButton.setEnabled(False)
            self.ui.RecButton.setText("")

        if self.lsb is not None and self.nb is not None:
            self.ui.RecButton.setEnabled(True)
            self.ui.RecButton.setText("Recover")

    def hide(self):
        """Hiding Driver Code."""
        global Infile, Datafile, Outfile

        self.ui.InfileDragDropLabel.setText("Hiding...")

        lsb = self.lsb if self.lsb in range(1, 8) else None
        steg = hide_data(Infile, Outfile, Datafile, lsb)
        if steg in ERRORCODES:
            self.ui.InfileDragDropLabel.setText(Error[code])
            return

        self.ui.InfileDragDropLabel.setText("Hiding Complete")
        self.ui.DatafileDragDropLabel.setText(f"LSB -> {steg[0]}")
        self.ui.SaveLocBrowseStegoLabel.setText(f"No. of bytes hidden -> {steg[1]}")
        self.ui.lsb.setText(f"{steg}")

    def stego_add_datadialog(self):
        """Add data file dialog for Stagnography page."""
        file = QFileDialog.getOpenFileName(
            caption="Data File")[0]
        if not file:
            self.ui.DatafileDragDropLabel.setText("Invalid File.")
            QtCore.QTimer.singleShot(
                2000, lambda: self.ui.DatafileDragDropLabel.setText("Click To Browse Save Location"))
            self.reset()

        else:
            self.stego_add_datafile(file)

    def stego_save_file(self):
        """Get Save File location for steganography page."""
        global Outfile

        file = QFileDialog.getSaveFileName(
            caption="Save File", filter="*.wav")[0]

        if not file or not file.endswith(".wav"):
            self.ui.CryptoSaveLocBrowseL.setText("Invalid Save Filename.")
            QtCore.QTimer.singleShot(
                2000, lambda: self.ui.CryptoSaveLocBrowseL.setText("Drag & Drop/\
                                                                   \nClick To Add File"))
        else:
            Outfile = file
            Size = 15
            self.ui.lsb.setMaximumSize(QtCore.QSize(16777215, 35))
            self.ui.lsb.setEnabled(True)
            font = QFont()
            font.setFamily("Feast of Flesh BB")
            font.setPointSize(Size)
            font.setBold(True)
            font.setItalic(True)
            font.setWeight(75)
            self.ui.SaveLocBrowseStegoLabel.setFont(font)
            self.ui.SaveLocBrowseStegoLabel.setText("Save To: \n" +
                                                    os.path.basename(file))
            self.ui.SaveLocBrowseStegoLabel.setStyleSheet(DragDropLabelSS2)
            self.ui.HideBtn.setEnabled(True)
            self.ui.HideBtn.setText("Hide")

    def stego_save_file_rec(self):
        """Save File Location for Stagnography page."""
        global Outfile, Datafile

        file = QFileDialog.getSaveFileName(
            caption="Save File")[0]

        if not file:
            self.ui.OutfileRecLabel.setText("Invalid Save Filename.")
            self.ui.SaveLocationBrowseLabel.setText("Invalid Save Filename.")
            QtCore.QTimer.singleShot(
                2000, lambda: self.ui.OutfileRecLabel.setText("Click To Browse Save Location"))
            QtCore.QTimer.singleShot(
                2000, lambda: self.ui.SaveLocationBrowseLabel.setText("Click To Browse Save Location"))

        else:
            Outfile = file
            font = QFont()
            font.setFamily("Feast of Flesh BB")
            font.setPointSize(15)
            font.setBold(True)
            font.setItalic(True)
            font.setWeight(75)
            self.ui.OutfileRecLabel.setFont(font)
            self.ui.SaveLocationBrowseLabel.setFont(font)
            self.ui.OutfileRecLabel.setText("Save To: \n" +
                                            os.path.basename(file))
            self.ui.SaveLocationBrowseLabel.setText("Save To: \n" +
                                                    os.path.basename(file))
            self.ui.OutfileRecLabel.setStyleSheet(DragDropLabelSS2)
            self.ui.SaveLocationBrowseLabel.setStyleSheet(DragDropLabelSS2)
            self.ui.RecButton.setEnabled(True)
            self.ui.RecButton.setText("Recover")
            self.ui.LSBNB.setMinimumSize(QtCore.QSize(0, 72))
            self.ui.LSBNB.setMaximumSize(QtCore.QSize(16777215, 72))
            self.ui.LSBNbytes.setMinimumSize(QtCore.QSize(0, 60))
            self.ui.LSBNbytes.setMaximumSize(QtCore.QSize(16777215, 60))
            if Datafile:
                self.ui.Forward.setEnabled(True)
                self.ui.Forward.setText("Forward Circuit[Enc -> Hide]")


    def stego_add_datafile(self, file):
        """Add data file for Stagnography Page."""
        global Datafile, Outfile

        Size = 15
        Datafile = file
        if file is None:
            return
        self.ui.MDatafileLabel.setStyleSheet(DragDropLabelSS2)
        self.ui.DatafileDragDropLabel.setStyleSheet(DragDropLabelSS2)
        if len(file) > 105:
            file = file[:102] + '...'
        font = QFont()
        font.setFamily("Feast of Flesh BB")
        font.setPointSize(Size)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        self.ui.DatafileDragDropLabel.setFont(font)
        self.ui.MDatafileLabel.setFont(font)
        self.ui.DatafileDragDropLabel.setText("File Loaded:\n" +
                                            os.path.basename(file))
        self.ui.MDatafileLabel.setText("File Loaded:\n" +
                                       os.path.basename(file))
        self.ui.SaveLocBrowseStego.setEnabled(True)
        self.ui.SaveLocationBrowseLabel.setEnabled(True)
        self.ui.SaveLocBrowseStegoLabel.setEnabled(True)
        self.ui.SaveLocBrowseStegoLabel.setText(
            "Click To Browse Save Location")
        if Outfile:
            self.ui.Forward.setEnabled(True)
            self.ui.Forward.setText("Forward Circuit[Enc -> Hide]")

    def stego_add_dialog(self):
        """Open file dialog for stego page."""
        file = QFileDialog.getOpenFileName(
            caption="Audio File", filter="*.wav")[0]
        if not file.endswith('.wav'):
            file = None
        if not file:
            self.ui.InfileDragDropLabel.setText("Invalid Audio.")
            QtCore.QTimer.singleShot(
                2000, lambda: self.ui.InfileDragDropLabel.setText("Click To Browse Save Location"))
            self.reset()

        else:
            self.stego_add_file(file)

    def stego_add_file(self, file):
        """Add Input File for stego page."""
        global Infile

        Size = 15
        Infile = file
        if not file:
            self.ui.InfileDragDropLabel.setText("Invalid File.")
            self.ui.InfileRecLabel.setText("Invalid File.")
            self.ui.MInfileLabel.setText("Invalid File.")
            QtCore.QTimer.singleShot(
                8000, self.reset())
        self.ui.InfileDragDropLabel.setStyleSheet(DragDropLabelSS2)
        self.ui.InfileRecLabel.setStyleSheet(DragDropLabelSS2)
        self.ui.MInfileLabel.setStyleSheet(DragDropLabelSS2)
        if len(file) > 105:
            file = file[:102] + '...'
        font = QFont()
        font.setFamily("Feast of Flesh BB")
        font.setPointSize(Size)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        self.ui.MInfileLabel.setFont(font)
        self.ui.MInfileLabel.setText("File Loaded:\n" +
                                     os.path.basename(file))
        self.ui.MDatafileDragDrop.setEnabled(True)
        self.ui.InfileDragDropLabel.setFont(font)
        self.ui.InfileDragDropLabel.setText("File Loaded:\n" +
                                            os.path.basename(file))
        self.ui.InfileRecLabel.setFont(font)
        self.ui.InfileRecLabel.setText("File Loaded:\n" +
                                        os.path.basename(file))
        self.ui.DatafileDragDropLabel.setEnabled(True)
        self.ui.DatafileDragDrop.setEnabled(True)
        self.ui.DatafileDragDropLabel.setText(
            "(Data File)\nClick To Add/Drag & Drop Files")
        self.ui.OutfileRecLabel.setEnabled(True)
        self.ui.OutfileRecBrowse.setEnabled(True)
        self.ui.OutfileRecLabel.setStyleSheet(DragDropLabelSS1)
        self.ui.OutfileRecLabel.setText("Click To Browse Save Location")
        self.ui.RecButton.setText("Browse Save Location First.")
        self.ui.DatafileDragDropLabel.setStyleSheet(DragDropLabelSS1)
        self.ui.SaveLocBrowseStegoLabel.setText("Add Data File First")
        self.ui.SaveLocBrowseStegoLabel.setStyleSheet("")
        self.ui.MDatafileLabel.setStyleSheet(DragDropLabelSS1)
        self.ui.MDatafileLabel.setText(
            "(Datafile)\nClick To Add/Drag&Drop Files")
        self.ui.SaveLocBrowseStegoLabel.setStyleSheet(
            DragDropLabelSS1)
        self.ui.SaveLocationBrowseLabel.setText(
            "Click To Browse Save Location")
        self.ui.SaveLocationBrowseLabel.setStyleSheet(DragDropLabelSS1)
        self.ui.SaveLocationBrowse.setEnabled(True)

    def transfer(self):
        """Transfer Driver Code."""
        global URL, Infile

        if URL:
            self.ui.DragDropLabelTransfer.setText("Downloading...")

            location = anon_download(URL)
            self.ui.DragDropLabelTransfer.setStyleSheet(
                    DragDropLabelTransferSS2)
            if type(location) == int:
                self.reset()
                self.ui.DragDropLabelTransfer.setText(
                    f"{Error[location]}")
            else:
                self.reset()
                self.ui.DragDropLabelTransfer.setText(f"File Saved to: \n{location}")

        else:
            self.ui.DragDropLabelTransfer.setText("Uploading...")

            filename = anon_upload(Infile)
            if type(filename) == int:
                self.reset()
                self.ui.DragDropLabelTransfer.setText(
                    f"{Error[filename]}")
            else:
                self.reset()
                self.ui.DragDropLabelTransfer.setText(
                    f"Succesfully Uploaded")
                self.ui.Url.setText(filename)
                self.ui.Url.setEnabled(True)

    def transfer_add_file(self, file):
        """Add input file for transfer page."""
        global Infile

        Size = 14
        Infile = file
        self.ui.DragDropLabelTransfer.setStyleSheet(DragDropLabelSS2)
        if len(file) > 105:
            file = (file[:105] + '\n' + file[105:])
        if len(file) > 210:
            file = (file[:105] + '\n' + file[105:207] + '...')
        if len(file) < 80:
            Size = 25
        font = QtGui.QFont()
        font.setFamily("DEADLY KILLERS DEMO")
        font.setPointSize(Size)
        font.setBold(True)
        font.setItalic(False)
        font.setWeight(75)
        self.ui.DragDropLabelTransfer.setFont(font)
        self.ui.DragDropLabelTransfer.setText("File Loaded:\n" +
                                              os.path.basename(file))
        self.ui.Url.setText("Click Transfer to get URL")
        self.ui.Url.setEnabled(False)
        self.ui.ButtonTransfer.setEnabled(True)
        self.ui.ButtonTransfer.setText("Transfer")

    def transfer_file_dialog(self):
        """Open File dialog for transfer page."""
        global Infile

        if not Infile:
            filename = QFileDialog.getOpenFileName(caption="Input File")
            if not filename[0]:
                self.ui.DragDropLabelTransfer.setText("Invalid File.")
                QtCore.QTimer.singleShot(
                    2000, lambda: self.ui.DragDropLabelTransfer.setText("Click To Add/\n"
                                                                      "Drag & Drop Files"))
            else:
                self.transfer_add_file(filename[0])

    def update_transfer_label(self):
        """Update transfer label according to the text."""
        global URL

        if Infile:
            return

        text = self.ui.Url.text()
        if verify(text):
            Size = 12
            self.ui.DragDropLabelTransfer.setStyleSheet(
                DragDropLabelTransferSS2)
            if len(text) > 87:
                text = (text[:87] + '\n' + text[87:])
            if len(text) > 174:
                text = (text[:174] + '\n' + text[174:258] + '...')
            if len(text) < 60:
                Size = 18
            URL = text
            font = QtGui.QFont()
            font.setFamily("Courgette")
            font.setPointSize(Size)
            font.setBold(True)
            font.setItalic(False)
            font.setWeight(75)
            self.ui.DragDropLabelTransfer.setFont(font)
            self.ui.DragDropLabelTransfer.setText("Download From:\n" +
                                                  text)
            self.ui.ButtonTransfer.setEnabled(True)
            self.ui.ButtonTransfer.setText("Transfer")
        else:
            if not text:
                self.reset()
            else:
                font = QtGui.QFont()
                font.setFamily("Courgette")
                font.setPointSize(35)
                font.setBold(True)
                font.setItalic(False)
                font.setWeight(75)
                self.ui.DragDropLabelTransfer.setFont(font)
                self.ui.DragDropLabelTransfer.setText("Invalid URL.")
            self.ui.ButtonTransfer.setEnabled(False)
            self.ui.ButtonTransfer.setText("Add File/Url First")

    def decrypt(self):
        """Decryption Driver Code."""
        global Infile, Outfile
        self.ui.CryptoDragDropLabel.setText("Decrypting...")

        code = decryptor(Infile, Outfile, self.ui.Password.text(),
                                  self.ui.AlgoSelect.currentText())
        if code in ERRORCODES:
            self.ui.CryptoDragDropLabel.setText(Error[code])
        self.reset()
        self.ui.CryptoDragDropLabel.setText("Succesfully Decrypted")

    def encrypt(self):
        """Encryption Driver Code."""
        global Infile, Outfile
        self.ui.CryptoDragDropLabel.setText("Encrypting...")

        code = encryptor(Infile, Outfile, self.ui.Password.text(),
                                  self.ui.AlgoSelect.currentText())
        if code in ERRORCODES:
            self.ui.CryptoDragDropLabel.setText(Error[code])

        tmp = Outfile
        self.reset()
        self.ui.CryptoDragDropLabel.setText("Succesfully Encrypted")
        self.ui.CryptoSaveLocBrowseL.setText(f"File Saved as \n{tmp}")

    def update_password(self):
        """Toggle the button according to password field."""
        if self.ui.Password.text() not in [None, ""]:
            self.ui.Encrypt.setEnabled(True)
            self.ui.Encrypt.setText("Encrypt")
            self.ui.Decrypt.setEnabled(True)
            self.ui.Decrypt.setText("Decrypt")

        else:
            self.ui.Encrypt.setEnabled(False)
            self.ui.Encrypt.setText("")
            self.ui.Decrypt.setEnabled(False)
            self.ui.Decrypt.setText("")

    def crypto_add_file(self, file):
        """Add input file for crypto page."""
        global Infile

        Size = 15
        Infile = file
        self.ui.CryptoDragDropLabel.setStyleSheet(DragDropLabelSS2)
        if len(file) > 105:
            file = file[:102] + '...'
        font = QFont()
        font.setFamily("Cabin Sketch")
        font.setPointSize(Size)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        self.ui.CryptoDragDropLabel.setFont(font)
        self.ui.CryptoDragDropLabel.setText("File Loaded:\n" +
                                            os.path.basename(file))
        self.ui.CryptoSaveLocBrowse.setEnabled(True)
        self.ui.CryptoSaveLoc.setEnabled(True)
        self.ui.CryptoSaveLocBrowseL.setText(
            "Click To Browse Save Location"
        )
        self.ui.CryptoSaveLocBrowseL.setStyleSheet(
            SaveLocationLabelSS2)

    def crypto_add_dialog(self):
        """Open File Dialog for Crypto Page."""
        file = QFileDialog.getOpenFileName(
            caption="Input File")[0]
        if not file:
            self.ui.CryptoSaveLocBrowseL.setText("Invalid Save Filename.")
            QtCore.QTimer.singleShot(
                2000, lambda: self.ui.CryptoSaveLocBrowseL.setText("Click To Browse Save Location"))
            self.reset()

        else:
            self.crypto_add_file(file)

    def crypto_save_file(self):
        """Get the output file for Crypto Page."""
        global Outfile

        file = QFileDialog.getSaveFileName(
            caption="Save File")[0]

        if not file:
            self.ui.CryptoSaveLocBrowseL.setText("Invalid Save Filename.")
            QtCore.QTimer.singleShot(
                2000, lambda: self.ui.CryptoSaveLocBrowseL.setText("Drag & Drop/\
                                                                   \nClick To Add File"))
        else:
            Outfile = file
            Size = 15
            self.ui.Password.setMaximumSize(QtCore.QSize(16777215, 28))
            self.ui.AlgoSelect.setEnabled(True)
            self.ui.AlgoSelect.setMaximumSize(QtCore.QSize(16777215, 25))
            font = QFont()
            font.setFamily("Cabin Sketch")
            font.setPointSize(Size)
            font.setBold(True)
            font.setItalic(True)
            font.setWeight(75)
            self.ui.CryptoSaveLocBrowseL.setFont(font)
            self.ui.CryptoSaveLocBrowseL.setText("Save To: " +
                                                  os.path.basename(file))
            self.ui.CryptoSaveLocBrowseL.setStyleSheet(DragDropLabelSS2)

    def reset(self):
        """Reset the GUI to initial state."""
        global Infile, Outfile, URL, Datafile

        Infile = None
        Outfile = None
        URL = None
        Datafile = None
        self.lsb = None
        self.nb = None

        # Reserved for an update.
        self.ui.StegEnc.setEnabled(False)

        # Crypto Page
        self.ui.CryptoSaveLoc.setEnabled(False)
        self.ui.CryptoSaveLocBrowseL.setText("Add Input File First")
        self.ui.CryptoSaveLocBrowseL.setStyleSheet(SaveLocationLabelSS1)
        self.ui.Password.setMaximumSize(QtCore.QSize(0, 0))
        self.ui.Encrypt.setEnabled(False)
        self.ui.Encrypt.setText("")
        self.ui.Decrypt.setEnabled(False)
        self.ui.Decrypt.setText("")
        self.ui.CryptoDragDropLabel.setStyleSheet(DragDropLabelSS1)
        self.ui.CryptoDragDropLabel.setText("Click To Add/\n"
                                            "Drag & Drop Files")
        self.ui.AlgoSelect.setEnabled(False)
        self.ui.AlgoSelect.setMaximumSize(QtCore.QSize(0, 0))
        font = QFont()
        font.setFamily("Cabin Sketch")
        font.setPointSize(15)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        self.ui.AlgoSelect.setFont(font)
        self.ui.Password.setText("")

        # Transfer Page
        self.ui.ButtonTransfer.setEnabled(False)
        self.ui.ButtonTransfer.setText("Add File/Url First")
        self.ui.Url.setEnabled(True)
        self.ui.Url.setText("")
        self.ui.DragDropLabelTransfer.setStyleSheet(DragDropLabelTransferSS1)
        font = QtGui.QFont()
        font.setFamily("DEADLY KILLERS DEMO")
        font.setPointSize(30)
        font.setBold(True)
        font.setItalic(False)
        font.setWeight(75)
        self.ui.DragDropLabelTransfer.setFont(font)
        self.ui.DragDropLabelTransfer.setText("Click To Add/\n"
                                              "Drag & Drop Files")
        self.ui.Url.setPlaceholderText("Paste URL Here.")

        # Hide Page
        self.ui.InfileDragDropLabel.setText("(Audio File)\nClick To Add/Drag & Drop Files")
        self.ui.InfileDragDropLabel.setStyleSheet(DragDropLabelSS1)
        self.ui.DatafileDragDrop.setEnabled(False)
        self.ui.DatafileDragDropLabel.setText("Add Input File First")
        self.ui.DatafileDragDropLabel.setStyleSheet("")
        self.ui.SaveLocBrowseStego.setEnabled(False)
        self.ui.SaveLocBrowseStegoLabel.setText("")
        self.ui.SaveLocBrowseStegoLabel.setStyleSheet("")
        self.ui.HideBtn.setEnabled(False)
        self.ui.HideBtn.setText("")
        self.ui.lsb.setMaximumSize(QtCore.QSize(0, 0))

        # Recover Page
        self.ui.InfileRecLabel.setStyleSheet(DragDropLabelSS1)
        self.ui.InfileRecLabel.setText(
            "(Stego Audio)\n Drag&Drop/Click To Add File")
        self.ui.OutfileRecBrowse.setEnabled(False)
        self.ui.OutfileRecLabel.setText("Add Input File First.")
        self.ui.OutfileRecLabel.setStyleSheet("")
        self.ui.LSBNB.setMaximumSize(QtCore.QSize(0, 0))
        self.ui.RecButton.setEnabled(False)
        self.ui.RecButton.setText("")

        # Macro EncSteg Page
        self.ui.Backward.setEnabled(False)
        self.ui.MInfileLabel.setStyleSheet(DragDropLabelSS1)
        self.ui.Backward.setText("")
        self.ui.Forward.setEnabled(False)
        self.ui.Forward.setText("")
        self.ui.LSBNbytes.setMaximumSize(QtCore.QSize(0, 0))
        self.ui.SaveLocationBrowse.setEnabled(False)
        self.ui.SaveLocationBrowseLabel.setStyleSheet("")
        self.ui.SaveLocationBrowseLabel.setText("")
        self.ui.MDatafileDragDrop.setEnabled(False)
        self.ui.MDatafileLabel.setStyleSheet("border: 0px solid;")
        self.ui.MDatafileLabel.setText("Add Audio File First.")
        self.ui.MInfileDragDrop.setEnabled(True)
        self.ui.MInfileLabel.setText(
            "(Audiofile)\nClick To Add/Drag&Drop Files")


# Splash Screen
class SplashScreen(QMainWindow):
    """Splash Screen."""

    def __init__(self):
        """Set up the splash screen."""
        QMainWindow.__init__(self)
        self.ui = Ui_SplashScreen()
        self.ui.setupUi(self)

        # Remove Window Frame & Make background translucent
        self.setWindowFlag(QtCore.Qt.FramelessWindowHint)
        self.setAttribute(QtCore.Qt.WA_TranslucentBackground)

        # Set Drop Shadow
        self.shadow = QGraphicsDropShadowEffect(self)
        self.shadow.setBlurRadius(5)
        self.shadow.setXOffset(0)
        self.shadow.setYOffset(0)
        self.shadow.setColor(QColor(0, 0, 0, 90))
        self.ui.DropShadowFrame.setGraphicsEffect(self.shadow)

        # Connecting The Progress Bar
        self.timer = QtCore.QTimer()
        self.timer.timeout.connect(self.load)
        self.timer.start(20)

        # Loading Screen Updates
        QtCore.QTimer.singleShot(
            1500, lambda: self.ui.LoadingStatus.setText("Importing Modules"))
        QtCore.QTimer.singleShot(
            2800, lambda: self.ui.LoadingStatus.setText("Almost Done..."))

        self.show()

    def load(self):
        """Update the progress bar."""
        global Progress

        # Updating Progress Bar
        self.ui.ProgressBar.setValue(Progress)
        # When Progress Bar Completes
        if Progress > 100:
            self.timer.stop()

            # Show Main Window
            self.main = MainWindow()
            self.close()

        if Progress > 80:
            QtCore.QTimer.singleShot(
                0, lambda: self.ui.AppDesc.setText("Made by Rakshan Sharma"))

        Progress += 1


# App Driver Code.
if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = SplashScreen()
    sys.exit(app.exec_())



from setuptools import setup, find_packages


def readme():
  with open('README.md', encoding='utf-8') as f:
    README = f.read()
    return README


setup(
  name='thoryvos',
  packages=find_packages(include=['thoryvos']),
  version='1.0.1',
  license='MIT',
  description='The All-In One Cryptographic Toolkit',
  long_description=readme(),
  long_description_content_type='text/markdown',
  author='Rakshan Sharma',
  author_email='rakshan793@gmail.com',
  url='https://github.com/freakingrocky/thoryvos',
  download_url='https://github.com/freakingrocky/thoryvos/build/dist/thoryvos-08.tar.gz',
  include_package_data=True,
  keywords=['Cryptography', 'Crypto', 'Audio Cryptography',
              'Stagongraphy', 'Steganographer', 'Audio Steganography',
              'Cryptographic Toolkit', 'File Sharing', 'Anonymous File Sharing'],
  install_requires=[
          'PySide2',
          'pycryptodome',
          'argparse',
          'termcolor',
          'PyWave',
          'SoundFile',
          'anonfile'
      ],
  classifiers=[
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'Intended Audience :: End Users/Desktop',
    'Intended Audience :: Other Audience',
    'Intended Audience :: Science/Research',
    'Topic :: Communications :: File Sharing',
    'Topic :: Multimedia :: Sound/Audio',
    'Topic :: Multimedia :: Sound/Audio :: Conversion',
    'Topic :: Security',
    'Topic :: Security :: Cryptography',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.10'],
  entry_points={
      "console_scripts": [
          "thoryvos=thoryvos.__main__:main",
          "thoryvos_cli=thoryvos.__main__:parse_cl"
      ]
  },
)
